@@ -28,26 +28,41 @@
 #include <gio/giotypes.h>
 
 #include <gio/gappinfo.h>
-#include <gio/gasyncresult.h>
+#include <gio/gapplication.h>
 #include <gio/gasyncinitable.h>
+#include <gio/gasyncresult.h>
 #include <gio/gbufferedinputstream.h>
 #include <gio/gbufferedoutputstream.h>
 #include <gio/gcancellable.h>
+#include <gio/gcharsetconverter.h>
 #include <gio/gcontenttype.h>
 #include <gio/gconverter.h>
 #include <gio/gconverterinputstream.h>
 #include <gio/gconverteroutputstream.h>
-#include <gio/gcharsetconverter.h>
+#include <gio/gcredentials.h>
 #include <gio/gdatainputstream.h>
 #include <gio/gdataoutputstream.h>
+#include <gio/gdbusaddress.h>
+#include <gio/gdbusauthobserver.h>
+#include <gio/gdbusconnection.h>
+#include <gio/gdbuserror.h>
+#include <gio/gdbusintrospection.h>
+#include <gio/gdbusmessage.h>
+#include <gio/gdbusmethodinvocation.h>
+#include <gio/gdbusnameowning.h>
+#include <gio/gdbusnamewatching.h>
+#include <gio/gdbusproxy.h>
+#include <gio/gdbusserver.h>
+#include <gio/gdbusutils.h>
 #include <gio/gdrive.h>
 #include <gio/gemblemedicon.h>
-#include <gio/gfile.h>
 #include <gio/gfileattribute.h>
 #include <gio/gfileenumerator.h>
+#include <gio/gfile.h>
 #include <gio/gfileicon.h>
 #include <gio/gfileinfo.h>
 #include <gio/gfileinputstream.h>
+#include <gio/gfileiostream.h>
 #include <gio/gfilemonitor.h>
 #include <gio/gfilenamecompleter.h>
 #include <gio/gfileoutputstream.h>
@@ -56,15 +71,14 @@
 #include <gio/gicon.h>
 #include <gio/ginetaddress.h>
 #include <gio/ginetsocketaddress.h>
+#include <gio/ginitable.h>
 #include <gio/ginputstream.h>
 #include <gio/gioenums.h>
 #include <gio/gioenumtypes.h>
 #include <gio/gioerror.h>
 #include <gio/giomodule.h>
-#include <gio/ginitable.h>
 #include <gio/gioscheduler.h>
 #include <gio/giostream.h>
-#include <gio/gfileiostream.h>
 #include <gio/gloadableicon.h>
 #include <gio/gmemoryinputstream.h>
 #include <gio/gmemoryoutputstream.h>
@@ -74,23 +88,25 @@
 #include <gio/gnetworkaddress.h>
 #include <gio/gnetworkservice.h>
 #include <gio/goutputstream.h>
+#include <gio/gpermission.h>
 #include <gio/gresolver.h>
 #include <gio/gseekable.h>
 #include <gio/gsettings.h>
 #include <gio/gsimpleasyncresult.h>
-#include <gio/gsocket.h>
-#include <gio/gsocketaddress.h>
+#include <gio/gsimplepermission.h>
 #include <gio/gsocketaddressenumerator.h>
+#include <gio/gsocketaddress.h>
 #include <gio/gsocketclient.h>
 #include <gio/gsocketconnectable.h>
 #include <gio/gsocketconnection.h>
 #include <gio/gsocketcontrolmessage.h>
+#include <gio/gsocket.h>
 #include <gio/gsocketlistener.h>
 #include <gio/gsocketservice.h>
-#include <gio/gtcpconnection.h>
-#include <gio/gthreadedsocketservice.h>
 #include <gio/gsrvtarget.h>
+#include <gio/gtcpconnection.h>
 #include <gio/gthemedicon.h>
+#include <gio/gthreadedsocketservice.h>
 #include <gio/gvfs.h>
 #include <gio/gvolume.h>
 #include <gio/gvolumemonitor.h>
@@ -100,3 +116,4 @@
 #undef __GIO_GIO_H_INSIDE__
 
 #endif /* __G_IO_H__ */
+
