@@ -1507,6 +1507,7 @@ g_settings_set (GSettings   *settings,
  * @mapping: the function to map the value in the settings database to
  *           the value used by the application
  * @user_data: user data for @mapping
+ * @returns: the result, which may be %NULL
  *
  * Gets the value that is stored at @key in @settings, subject to
  * application-level validation/mapping.
@@ -1960,13 +1961,34 @@ g_settings_get_has_unapplied (GSettings *settings)
            G_DELAYED_SETTINGS_BACKEND (settings->priv->backend));
 }
 
-/* Extra API (sync, get_child, is_writable, list_items) {{{1 */
+/* Extra API (reset, sync, get_child, is_writable, list_items) {{{1 */
+/**
+ * g_settings_reset:
+ * @settings: a #GSettings object
+ * @key: the name of a key
+ *
+ * Resets @key to its default value.
+ *
+ * This call resets the key, as much as possible, to its default value.
+ * That might the value specified in the schema or the one set by the
+ * administrator.
+ **/
+void
+g_settings_reset (GSettings *settings,
+                  const gchar *key)
+{
+  gchar *path;
+
+  path = g_strconcat (settings->priv->path, key, NULL);
+  g_settings_backend_reset (settings->priv->backend, path, NULL);
+  g_free (path);
+}
+
 /**
  * g_settings_sync:
- * @context: the context to sync, or %NULL
  *
- * Ensures that all pending operations for the given context are
- * complete.
+ * Ensures that all pending operations for the given are complete for
+ * the default backend.
  *
  * Writes made to a #GSettings are handled asynchronously.  For this
  * reason, it is very unlikely that the changes have it to disk by the
@@ -2056,7 +2078,7 @@ g_settings_get_child (GSettings   *settings,
 /**
  * g_settings_list_items:
  * @settings: a #GSettings object
- * Returns: a list of the keys on @settings
+ * @returns: a list of the keys on @settings
  *
  * Introspects the list of keys and children on @settings.
  *
@@ -2263,6 +2285,23 @@ g_settings_binding_property_changed (GObject          *object,
   binding->running = FALSE;
 }
 
+static gboolean
+g_settings_bind_invert_boolean_get_mapping (GValue   *value,
+                                            GVariant *variant,
+                                            gpointer  user_data)
+{
+  g_value_set_boolean (value, !g_variant_get_boolean (variant));
+  return TRUE;
+}
+
+static GVariant *
+g_settings_bind_invert_boolean_set_mapping (const GValue       *value,
+                                            const GVariantType *expected_type,
+                                            gpointer            user_data)
+{
+  return g_variant_new_boolean (!g_value_get_boolean (value));
+}
+
 /**
  * g_settings_bind:
  * @settings: a #GSettings object
@@ -2301,8 +2340,20 @@ g_settings_bind (GSettings          *settings,
                  const gchar        *property,
                  GSettingsBindFlags  flags)
 {
-  g_settings_bind_with_mapping (settings, key, object, property,
-                                flags, NULL, NULL, NULL, NULL);
+  GSettingsBindGetMapping get_mapping = NULL;
+  GSettingsBindSetMapping set_mapping = NULL;
+
+  if (flags & G_SETTINGS_BIND_INVERT_BOOLEAN)
+    {
+      get_mapping = g_settings_bind_invert_boolean_get_mapping;
+      set_mapping = g_settings_bind_invert_boolean_set_mapping;
+
+      /* can't pass this flag to g_settings_bind_with_mapping() */
+      flags &= ~G_SETTINGS_BIND_INVERT_BOOLEAN;
+    }
+
+  g_settings_bind_with_mapping (settings, key, object, property, flags,
+                                get_mapping, set_mapping, NULL, NULL);
 }
 
 /**
@@ -2349,6 +2400,7 @@ g_settings_bind_with_mapping (GSettings               *settings,
   GQuark binding_quark;
 
   g_return_if_fail (G_IS_SETTINGS (settings));
+  g_return_if_fail (~flags & G_SETTINGS_BIND_INVERT_BOOLEAN);
 
   objectclass = G_OBJECT_GET_CLASS (object);
 
@@ -2386,10 +2438,39 @@ g_settings_bind_with_mapping (GSettings               *settings,
       return;
     }
 
-  if (((get_mapping == NULL && (flags & G_SETTINGS_BIND_GET)) ||
-       (set_mapping == NULL && (flags & G_SETTINGS_BIND_SET))) &&
-      !g_settings_mapping_is_compatible (binding->property->value_type,
-                                         binding->info.type))
+  if (get_mapping == g_settings_bind_invert_boolean_get_mapping)
+    {
+      /* g_settings_bind_invert_boolean_get_mapping() is a private
+       * function, so if we are here it means that g_settings_bind() was
+       * called with G_SETTINGS_BIND_INVERT_BOOLEAN.
+       *
+       * Ensure that both sides are boolean.
+       */
+
+      if (binding->property->value_type != G_TYPE_BOOLEAN)
+        {
+          g_critical ("g_settings_bind: G_SETTINGS_BIND_INVERT_BOOLEAN "
+                      "was specified, but property `%s' on type `%s' has "
+                      "type `%s'", property, G_OBJECT_TYPE_NAME (object),
+                      g_type_name ((binding->property->value_type)));
+          return;
+        }
+
+      if (!g_variant_type_equal (binding->info.type, G_VARIANT_TYPE_BOOLEAN))
+        {
+          g_critical ("g_settings_bind: G_SETTINGS_BIND_INVERT_BOOLEAN "
+                      "was specified, but key `%s' on schema `%s' has "
+                      "type `%s'", key, settings->priv->schema_name,
+                      g_variant_type_dup_string (binding->info.type));
+          return;
+        }
+
+    }
+
+  else if (((get_mapping == NULL && (flags & G_SETTINGS_BIND_GET)) ||
+            (set_mapping == NULL && (flags & G_SETTINGS_BIND_SET))) &&
+           !g_settings_mapping_is_compatible (binding->property->value_type,
+                                              binding->info.type))
     {
       g_critical ("g_settings_bind: property '%s' on class '%s' has type "
                   "'%s' which is not compatible with type '%s' of key '%s' "
