@@ -38,7 +38,6 @@
 #include <gio/gioerror.h>
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gsimpleasyncresult
@@ -836,6 +835,3 @@ g_simple_async_report_gerror_in_idle (GObject *object,
   g_simple_async_result_complete_in_idle (simple);
   g_object_unref (simple);
 }
-
-#define __G_SIMPLE_ASYNC_RESULT_C__
-#include "gioaliasdef.c"
